package dev.svitan.plugins

import dev.svitan.schemas.UserDTO
import dev.svitan.schemas.UserService
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import org.jetbrains.exposed.sql.Database
import io.github.cdimascio.dotenv.Dotenv

fun Application.configureDatabases(dotenv: Dotenv) {
    val dbHost = dotenv["DB_HOST"] ?: throw Exception("DB_HOST not found")
    val dbPort = dotenv["DB_PORT"] ?: throw Exception("DB_PORT not found")
    val dbName = dotenv["DB_NAME"] ?: throw Exception("DB_NAME not found")
    val dbUser = dotenv["DB_USER"] ?: throw Exception("DB_USER not found")
    val dbPassword = dotenv["DB_PASSWORD"] ?: throw Exception("DB_PASSWORD not found")

    Database.connect(
        url = "jdbc:postgresql://${dbHost}:${dbPort}/${dbName}",
        driver = "org.postgresql.Driver",
        user = dbUser,
        password = dbPassword
    )
    val userService = UserService()

    routing {
        // Create user
        post("/users") {
            val user = call.receive<UserDTO>()
            val id = userService.create(user)
            call.respond(HttpStatusCode.Created, id)
        }

        // Read user
        get("/users/{id}") {
            val id = call.parameters["id"]?.toInt() ?: throw IllegalArgumentException("Invalid ID")
            val user = userService.read(id)
            if (user != null) {
                call.respond(HttpStatusCode.OK, user)
            } else {
                call.respond(HttpStatusCode.NotFound)
            }
        }

        // Update user
        put("/users/{id}") {
            val id = call.parameters["id"]?.toInt() ?: throw IllegalArgumentException("Invalid ID")
            val user = call.receive<UserDTO>()
            userService.update(id, user)
            call.respond(HttpStatusCode.OK)
        }

        // Delete user
        delete("/users/{id}") {
            val id = call.parameters["id"]?.toInt() ?: throw IllegalArgumentException("Invalid ID")
            userService.delete(id)
            call.respond(HttpStatusCode.OK)
        }
    }
}
