package dev.svitan.schemas

import kotlinx.serialization.Serializable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction

@Serializable
data class UserDTO(val name: String, val age: Int)

class UserService {
    object Users : Table("users") {
        val id = integer("id").autoIncrement()
        val name = varchar("name", length = 50)
        val age = integer("age")

        override val primaryKey = PrimaryKey(id)
    }

    init {
        transaction {
            SchemaUtils.create(Users)
        }
    }

    fun create(user: UserDTO): Int =
        transaction {
            Users.insert {
                it[name] = user.name
                it[age] = user.age
            }[Users.id]
        }

    fun read(id: Int): UserDTO? =
        transaction {
            Users.selectAll()
                .where { Users.id eq id }
                .map { UserDTO(it[Users.name], it[Users.age]) }
                .singleOrNull()
        }

    fun update(id: Int, user: UserDTO) {
        transaction {
            Users.update({ Users.id eq id }) {
                it[name] = user.name
                it[age] = user.age
            }
        }
    }

    fun delete(id: Int) {
        transaction {
            Users.deleteWhere { Users.id.eq(id) }
        }
    }
}
